@@ -12,19 +12,23 @@
 #'
 #' @return Returns a binary matrix with mutated genes. (matrix)
 #'
-#' @examples {
+#' @examples
 #' ## Create 4 random individuals with binary values
 #' a <- cbind(bin=sample(c(0,1),20,replace=TRUE,prob = c(70,30)),
 #'         bin.1=sample(c(0,1),20,replace=TRUE,prob = c(30,70)),
 #'         bin.2=sample(c(0,1),20,replace=TRUE,prob = c(30,70)),
 #'         bin.3=sample(c(0,1),20,replace=TRUE,prob = c(30,70)))
 #' a
-#' ## Mutate the individuals with a given percentage
-#' aMut <- mutation(a,0.1)
 #'
+#' ## Mutate the individuals with a low percentage
+#' aMut <- mutation(a,0.1)
+#' ## Check which values are not like the originals
+#' a==aMut
+#'
+#' ## Mutate the individuals with a high percentage
+#' aMut <- mutation(a,0.4)
 #' ## Check which values are not like the originals
 #' a==aMut
-#' }
 #'
 #' @author Sebastian Gatscha
 mutation          <- function(a,p) {
@@ -35,7 +39,7 @@ mutation          <- function(a,p) {
     ## If the random number is smaller than p (0.1), the gene is switched
     ## (from 1 to 0 | from 0 to 1)
     if (rnd < p) {
-      if (a[i] == "0") {a[i] = 1} else {a[i] =1}
+      if (a[i] == "0") {a[i] <- 1} else {a[i] <- 1}
     }
   }
   return(a)
